package djmil.cordacheckers.cordaclient;

import org.junit.jupiter.api.Test;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;

import djmil.cordacheckers.cordaclient.dao.Color;
import djmil.cordacheckers.cordaclient.dao.GameProposal;
import djmil.cordacheckers.cordaclient.dao.VirtualNode;
import djmil.cordacheckers.cordaclient.dao.flow.arguments.GameProposalActionReq.Action;
import djmil.cordacheckers.user.HoldingIdentityResolver;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;

import java.util.List;
import java.util.UUID;

import javax.naming.InvalidNameException;

@SpringBootTest
public class CordaClientTest {
    @Autowired
    CordaClient cordaClient;

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    @Test
    void testGetVirtualNodeList() throws InvalidNameException {
        List<VirtualNode> vNodes = cordaClient.getVirtualNodeList();

		assertThat(vNodes.size()).isEqualTo(5); // default vNode config for CSDE
    }

    @Test
    void testGameProposalList() throws JsonProcessingException {
        List<GameProposal> gpList = cordaClient.gameProposalList(
            holdingIdentityResolver.getByUsername("Alice"));

        System.out.println("testListGameProposals\n"+ gpList);
    }

    @Test
    void testGemeProposalCreate() throws JsonMappingException, JsonProcessingException {
        final String gpSender = "alice";
        final String gpReceiver = "bob";
        final Color gpReceiverColor = Color.WHITE;
        final String gpMessage = "GameProposal create test";

        final UUID createdGpUuid = cordaClient.gameProposalCreate(
            holdingIdentityResolver.getByUsername(gpSender),
            holdingIdentityResolver.getByUsername(gpReceiver),
            gpReceiverColor,
            gpMessage
        );

        List<GameProposal> gpListSender = cordaClient.gameProposalList(
            holdingIdentityResolver.getByUsername(gpSender));

        assertThat(findByUuid(gpListSender, createdGpUuid)).isNotNull();

        List<GameProposal> gpListReceiver = cordaClient.gameProposalList(
            holdingIdentityResolver.getByUsername(gpReceiver));

        assertThat(findByUuid(gpListReceiver, createdGpUuid)).isNotNull();
    }

    @Test
    void testGemeProposalReject() throws JsonMappingException, JsonProcessingException {
        final String gpSender = "alice";
        final String gpReceiver = "bob";
        final Color gpReceiverColor = Color.WHITE;
        final String gpMessage = "GameProposal REJECT test";

        final UUID gpUuid = cordaClient.gameProposalCreate(
            holdingIdentityResolver.getByUsername(gpSender),
            holdingIdentityResolver.getByUsername(gpReceiver),
            gpReceiverColor,
            gpMessage
        );

        System.out.println("Create GP UUID "+ gpUuid);

        assertThatThrownBy(() -> {
            cordaClient.gameProposalAction(
                holdingIdentityResolver.getByUsername(gpSender),
                gpUuid,
                Action.REJECT);
        });

        final String rejectRes = cordaClient.gameProposalAction(
            holdingIdentityResolver.getByUsername(gpReceiver),
            gpUuid,
            Action.REJECT
        );

        assertThat(rejectRes).isEqualToIgnoringCase("REJECTED");

        List<GameProposal> gpListSender = cordaClient.gameProposalList(
            holdingIdentityResolver.getByUsername(gpSender));

        assertThat(findByUuid(gpListSender, gpUuid)).isNull();

        List<GameProposal> gpListReceiver = cordaClient.gameProposalList(
            holdingIdentityResolver.getByUsername(gpReceiver));

        assertThat(findByUuid(gpListReceiver, gpUuid)).isNull();
    }

    private GameProposal findByUuid(List<GameProposal> gpList, UUID uuid) {
        for (GameProposal gameProposal : gpList) {
            if (gameProposal.id().compareTo(uuid) == 0)
                return gameProposal;
        };
        return null;
    }
}
