package djmil.cordacheckers.states;

import java.security.PublicKey;
import java.util.List;

import djmil.cordacheckers.contracts.GameProposalContract;
import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.base.annotations.CordaSerializable;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.BelongsToContract;
import net.corda.v5.ledger.utxo.ContractState;

@BelongsToContract(GameProposalContract.class)
public class GameProposalResolutionState implements ContractState {

    @CordaSerializable
    public enum Resolution {
        ACCEPT,
        REJECT,
        CANCEL
    }

    public final Resolution outcome;
    public final List<PublicKey> participants;

    public GameProposalResolutionState(
        Resolution outcome,
        GameProposalState gameProposal
    ) {
        this.outcome = outcome;
        this.participants = gameProposal.getParticipants();
    }

    @ConstructorForDeserialization
    public GameProposalResolutionState(
        Resolution outcome,
        List<PublicKey> participants
    ) {
        this.outcome = outcome;
        this.participants = participants;
    }

    public Resolution getOutcome() {
        return outcome;
    }

    public List<PublicKey> getParticipants() {
        return this.participants;
    }

    public MemberX500Name getRecipient(GameProposalState gameProposal) {
        switch (outcome) {
            case ACCEPT:
            case REJECT:
                return gameProposal.getSender();
            case CANCEL:
                return gameProposal.getRecipient();
            default:
                throw new RuntimeException("Unknown Resolution value: "+outcome.toString());
        }
    }
    
}
